@@ -7,7 +7,7 @@ from simplejson.scanner import JSONDecodeError
 LOGGER = singer.get_logger()
 
 
-def process(messages, config, recordHandlerFunc, stateHandlerFunc, schemaHandlerFunc, onCompleteHandler, exceptionHandlerFunc):
+def process(messages, config, recordHandlerFunc, stateHandlerFunc, schemaHandlerFunc, onCompleteHandler, onMetadataHandler, exceptionHandlerFunc):
 
     state = None
     schemas = {}
@@ -17,7 +17,12 @@ def process(messages, config, recordHandlerFunc, stateHandlerFunc, schemaHandler
     try:
         for message in messages:
             try:
-                o = singer.parse_message(message).asdict()
+                # o = singer.parse_message(message).asdict()
+                """
+                singer.parse_message is doing more that just loading the Json message asdict
+                Since we added a new (non-singer) METADATA message, we need to be able to load any message types
+                """
+                o = json.loads(message)
             except json.decoder.JSONDecodeError:
                 LOGGER.error("Unable to parse:\n{}".format(message))
                 raise
@@ -38,6 +43,12 @@ def process(messages, config, recordHandlerFunc, stateHandlerFunc, schemaHandler
                 record = o['record']
                 recordHandlerFunc(record, config)  # triggers callback in client app
 
+            elif message_type == 'METADATA':  # METADATA message
+
+                metadata = o['metadata']
+                LOGGER.debug('Received metadata: {}'.format(metadata))
+                onMetadataHandler(metadata, config)
+
             elif message_type == 'STATE':  # STATE message
                 LOGGER.debug('Setting state to {}'.format(o['value']))
                 state = o['value']
